"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DVCClient = void 0;
const Variable_1 = require("./Variable");
const Request_1 = require("./Request");
const Store_1 = __importDefault(require("./Store"));
const User_1 = require("./User");
const EventQueue_1 = require("./EventQueue");
const utils_1 = require("./utils");
const EventEmitter_1 = require("./EventEmitter");
const RequestConsolidator_1 = require("./RequestConsolidator");
class DVCClient {
    constructor(environmentKey, user, options) {
        this.store = new Store_1.default(window.localStorage);
        this.user = user;
        this.options = options;
        this.environmentKey = environmentKey;
        this.variableDefaultMap = {};
        this.eventQueue = new EventQueue_1.EventQueue(environmentKey, this, options === null || options === void 0 ? void 0 : options.flushEventsMS);
        this.requestConsolidator = new RequestConsolidator_1.RequestConsolidator();
        this.eventEmitter = new EventEmitter_1.EventEmitter();
        this.store.saveUser(this.user)
            .then(() => console.log('Successfully saved user to local storage!'));
        this.onInitialized = (0, Request_1.getConfigJson)(environmentKey, this.user)
            .then((config) => {
            const oldConfig = this.config;
            this.config = config;
            this.store.saveConfig(config)
                .then(() => console.log('Successfully saved config to local storage'));
            this.eventEmitter.emitInitialized(true);
            this.eventEmitter.emitFeatureUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.features) || {}, this.config.features);
            this.eventEmitter.emitVariableUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.variables) || {}, this.config.variables, this.variableDefaultMap);
            return this;
        })
            .catch((err) => {
            this.eventEmitter.emitInitialized(false);
            this.eventEmitter.emitError(err);
            return this;
        });
    }
    onClientInitialized(onInitialized) {
        if (onInitialized && typeof onInitialized === 'function') {
            this.onInitialized
                .then(() => onInitialized(null, this))
                .catch((err) => onInitialized(err));
            return;
        }
        return this.onInitialized;
    }
    variable(key, defaultValue) {
        var _a, _b;
        const defaultValueKey = typeof defaultValue === 'string' ? defaultValue : JSON.stringify(defaultValue);
        if (this.variableDefaultMap[key] && this.variableDefaultMap[key][defaultValueKey]) {
            return this.variableDefaultMap[key][defaultValueKey];
        }
        const data = Object.assign({ key,
            defaultValue }, (_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.variables) === null || _b === void 0 ? void 0 : _b[key]);
        const variable = new Variable_1.DVCVariable(data);
        this.variableDefaultMap[key] = Object.assign({ [defaultValueKey]: variable }, this.variableDefaultMap[key]);
        try {
            this.eventQueue.queueAggregateEvent({
                type: variable.isDefaulted
                    ? EventQueue_1.EventTypes.variableDefaulted
                    : EventQueue_1.EventTypes.variableEvaluated,
                target: variable.key,
            });
        }
        catch (e) {
            this.eventEmitter.emitError(e);
            console.log('Error with queueing aggregate events', e);
        }
        return variable;
    }
    identifyUser(user, callback) {
        const promise = new Promise((resolve, reject) => {
            try {
                this.eventQueue.flushEvents();
                let updatedUser;
                if (user.user_id === this.user.user_id) {
                    updatedUser = this.user.updateUser(user);
                }
                else {
                    updatedUser = new User_1.DVCPopulatedUser(user);
                }
                const oldConfig = this.config || {};
                this.onInitialized.then(() => this.requestConsolidator.queue('identify', (0, Request_1.getConfigJson)(this.environmentKey, updatedUser))).then((config) => {
                    this.config = config;
                    this.store.saveConfig(config)
                        .then(() => console.log('Successfully saved config to local storage'));
                    const oldFeatures = oldConfig.features || {};
                    const oldVariables = oldConfig.variables || {};
                    this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, this.variableDefaultMap);
                    return config.variables;
                }).then((variables) => {
                    this.user = updatedUser;
                    this.store.saveUser(updatedUser)
                        .then(() => console.log('Successfully saved user to local storage!'));
                    return resolve(variables || {});
                }).catch((err) => Promise.reject(err));
            }
            catch (err) {
                this.eventEmitter.emitError(err);
                reject(err);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then((variables) => callback(null, variables))
                .catch((err) => callback(err, null));
            return;
        }
        return promise;
    }
    resetUser(callback) {
        const anonUser = new User_1.DVCPopulatedUser({ isAnonymous: true });
        const promise = new Promise((resolve, reject) => {
            try {
                this.eventQueue.flushEvents();
                const oldConfig = this.config || {};
                this.onInitialized.then(() => this.requestConsolidator.queue('identify', (0, Request_1.getConfigJson)(this.environmentKey, anonUser))).then((config) => {
                    this.config = config;
                    this.user = anonUser;
                    this.store.saveConfig(config).then(() => {
                        console.log('Successfully saved config to local storage');
                    });
                    this.store.saveUser(anonUser).then(() => {
                        console.log('Successfully saved user to local storage!');
                    });
                    const oldFeatures = oldConfig.features || {};
                    const oldVariables = oldConfig.variables || {};
                    this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, this.variableDefaultMap);
                    resolve(config.variables || {});
                }).catch((e) => {
                    reject(e);
                });
            }
            catch (e) {
                this.eventEmitter.emitError(e);
                reject(e);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then((variables) => callback(null, variables))
                .catch((err) => callback(err, null));
            return;
        }
        return promise;
    }
    allFeatures() {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.features) || {};
    }
    allVariables() {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.variables) || {};
    }
    subscribe(key, handler) {
        this.eventEmitter.subscribe(key, handler);
    }
    unsubscribe(key, handler) {
        this.eventEmitter.unsubscribe(key, handler);
    }
    track(event) {
        (0, utils_1.checkParamDefined)('type', event.type);
        this.onInitialized.then(() => {
            this.eventQueue.queueEvent(event);
        });
    }
    flushEvents(callback) {
        return this.eventQueue.flushEvents().then(() => callback === null || callback === void 0 ? void 0 : callback());
    }
}
exports.DVCClient = DVCClient;
//# sourceMappingURL=Client.js.map